==> ausweisapp2_1.26.5.orig.tar.gz: AusweisApp2-1.26.5/src/card/smart/SmartReaderManagerPlugIn.cpp <==
/**
 * Copyright (c) 2021-2023 Governikus GmbH & Co. KG, Germany
 */

#include "SmartReaderManagerPlugIn.h"

#include "Env.h"
#include "SmartManager.h"
#include "VolatileSettings.h"

#include <QLoggingCategory>


using namespace governikus;


Q_DECLARE_LOGGING_CATEGORY(card_smart)


bool SmartReaderManagerPlugIn::initializeSmart(const QSharedPointer<SmartManager>& pSmartManager) const
{
	const auto& smartAvailable = isSmartAvailable(pSmartManager);
	if (Env::getSingleton<VolatileSettings>()->isUsedAsSDK() || smartAvailable)
	{
		return smartAvailable;
	}

	qCDebug(card_smart) << "Updating Smart-eID info";
	pSmartManager->updateInfo();

	return isSmartAvailable(pSmartManager);
}


bool SmartReaderManagerPlugIn::isSmartAvailable(const QSharedPointer<SmartManager>& pSmartManager) const
{
	const auto& eidStatus = pSmartManager->status();
	return eidStatus != EidStatus::INTERNAL_ERROR && eidStatus != EidStatus::UNAVAILABLE;
}


SmartReaderManagerPlugIn::SmartReaderManagerPlugIn()
	: ReaderManagerPlugIn(ReaderManagerPlugInType::SMART)
	, mSmartReader(nullptr)
{
}


QList<Reader*> SmartReaderManagerPlugIn::getReaders() const
{
	if (mSmartReader)
	{
		return {mSmartReader.data()};
	}

	return {};
}


void SmartReaderManagerPlugIn::init()
{
	if (getInfo().isAvailable())
	{
		return;
	}

	ReaderManagerPlugIn::init();

	const auto& smartManager = SmartManager::get();
	if (!Env::getSingleton<VolatileSettings>()->isUsedAsSDK())
	{
		smartManager->abortSDKWorkflow();
	}

	if (!initializeSmart(smartManager))
	{
		qCWarning(card_smart) << "Smart-eID was not initialized";
		return;
	}

	setPlugInAvailable(true);
	mSmartReader.reset(new SmartReader());
	connect(mSmartReader.data(), &SmartReader::fireReaderPropertiesUpdated, this, &SmartReaderManagerPlugIn::fireReaderPropertiesUpdated);
	connect(mSmartReader.data(), &SmartReader::fireCardInfoChanged, this, &SmartReaderManagerPlugIn::fireCardInfoChanged);
	connect(mSmartReader.data(), &SmartReader::fireCardInserted, this, &SmartReaderManagerPlugIn::fireCardInserted);
	connect(mSmartReader.data(), &SmartReader::fireCardRemoved, this, &SmartReaderManagerPlugIn::fireCardRemoved);
	qCDebug(card_smart) << "Add reader" << mSmartReader->getName();
	Q_EMIT fireReaderAdded(mSmartReader->getReaderInfo());
}


void SmartReaderManagerPlugIn::shutdown()
{
	if (!getInfo().isAvailable())
	{
		return;
	}

	setPlugInAvailable(false);
	mSmartReader.reset();
}


void SmartReaderManagerPlugIn::insert(const QString& pReaderName, const QVariant& pData)
{
	if (!getInfo().isAvailable())
	{
		qCDebug(card_smart) << "Skipping insert because Smart-eID is not available on this device";
		return;
	}

	if (!isScanRunning())
	{
		qCDebug(card_smart) << "Skipping insert because the scan is not running";
		return;
	}

	if (mSmartReader->getReaderInfo().getName() != pReaderName)
	{
		qCDebug(card_smart) << "Skipping insert because of an unexpected reader name";
		return;
	}

	mSmartReader->insertCard(pData);
}


void SmartReaderManagerPlugIn::startScan(bool pAutoConnect)
{
	if (getInfo().isAvailable())
	{
		mSmartReader->connectReader();
		ReaderManagerPlugIn::startScan(pAutoConnect);
	}
}


void SmartReaderManagerPlugIn::stopScan(const QString& pError)
{
	if (getInfo().isAvailable())
	{
		mSmartReader->disconnectReader(pError);
		ReaderManagerPlugIn::stopScan(pError);
	}
}

==> ausweisapp2_2.0.3.orig.tar.gz: AusweisApp-2.0.3/src/card/smart/SmartReaderManagerPlugIn.cpp <==
/**
 * Copyright (c) 2021-2023 Governikus GmbH & Co. KG, Germany
 */

#include "SmartReaderManagerPlugIn.h"

#include "AppSettings.h"
#include "SmartManager.h"
#include "VolatileSettings.h"

#include <QLoggingCategory>


using namespace governikus;


Q_DECLARE_LOGGING_CATEGORY(card_smart)


static QString READER_NAME()
{
	return QStringLiteral("Smart");
}


void SmartReaderManagerPlugIn::publishReader(const ReaderInfo& pInfo)
{
	if (mReaderAdded)
	{
		Q_EMIT fireReaderPropertiesUpdated(pInfo);
		return;
	}

	Q_EMIT fireReaderAdded(pInfo);
	mReaderAdded = true;
}


void SmartReaderManagerPlugIn::onSmartAvailableChanged(bool pSmartAvailable)
{
	if (pSmartAvailable)
	{
		init();
		return;
	}

	shutdown();
}


SmartReaderManagerPlugIn::SmartReaderManagerPlugIn()
	: ReaderManagerPlugIn(ReaderManagerPlugInType::SMART)
	, mReaderAdded(false)
	, mSmartReader(nullptr)
{
	connect(&Env::getSingleton<AppSettings>()->getGeneralSettings(), &GeneralSettings::fireSmartAvailableChanged, this, &SmartReaderManagerPlugIn::onSmartAvailableChanged);
}


QList<Reader*> SmartReaderManagerPlugIn::getReaders() const
{
	if (mSmartReader)
	{
		return {mSmartReader.data()};
	}

	return {};
}


void SmartReaderManagerPlugIn::init()
{
	ReaderManagerPlugIn::init();

	const auto& smartManager = SmartManager::get();
	if (!Env::getSingleton<VolatileSettings>()->isUsedAsSDK())
	{
		smartManager->abortSDKWorkflow();
	}

	if (!smartManager->smartAvailable())
	{
		qCWarning(card_smart) << "Smart-eID is not available";
		publishReader(ReaderInfo(READER_NAME()));
		return;
	}

	if (getInfo().isAvailable())
	{
		return;
	}

	setPlugInAvailable(true);
	mSmartReader.reset(new SmartReader(READER_NAME()));
	connect(mSmartReader.data(), &SmartReader::fireReaderPropertiesUpdated, this, &SmartReaderManagerPlugIn::fireReaderPropertiesUpdated);
	connect(mSmartReader.data(), &SmartReader::fireCardInfoChanged, this, &SmartReaderManagerPlugIn::fireCardInfoChanged);
	connect(mSmartReader.data(), &SmartReader::fireCardInserted, this, &SmartReaderManagerPlugIn::fireCardInserted);
	connect(mSmartReader.data(), &SmartReader::fireCardRemoved, this, &SmartReaderManagerPlugIn::fireCardRemoved);
	qCDebug(card_smart) << "Add reader" << mSmartReader->getName();
	publishReader(mSmartReader->getReaderInfo());
}


void SmartReaderManagerPlugIn::shutdown()
{
	if (!getInfo().isAvailable())
	{
		return;
	}

	mSmartReader.reset();
	Q_EMIT fireReaderPropertiesUpdated(ReaderInfo(READER_NAME()));
	setPlugInAvailable(false);
}


void SmartReaderManagerPlugIn::insert(const QString& pReaderName, const QVariant& pData)
{
	if (!getInfo().isAvailable())
	{
		qCDebug(card_smart) << "Skipping insert because Smart-eID is not available on this device";
		return;
	}

	if (!getInfo().isScanRunning())
	{
		qCDebug(card_smart) << "Skipping insert because the scan is not running";
		return;
	}

	if (mSmartReader->getReaderInfo().getName() != pReaderName)
	{
		qCDebug(card_smart) << "Skipping insert because of an unexpected reader name";
		return;
	}

	mSmartReader->insertCard(pData);
}


void SmartReaderManagerPlugIn::startScan(bool pAutoConnect)
{
	if (getInfo().isAvailable())
	{
		mSmartReader->connectReader();
		ReaderManagerPlugIn::startScan(pAutoConnect);
	}
}


void SmartReaderManagerPlugIn::stopScan(const QString& pError)
{
	if (getInfo().isAvailable())
	{
		mSmartReader->disconnectReader(pError);
		ReaderManagerPlugIn::stopScan(pError);
	}
}
